#include "mbed.h"
#include "main_init.h"
#include "i2c_mbed_fpga.h"
#include "uart_mbed.h"
#include "scpi-def.h"

#define RAM_END_ADDR 0x4B

//#define CLIM_LEVEL 47       //1.5A
//#define CLIM_LEVEL 57       //1.8A
//#define CLIM_LEVEL 67       //2.1A
//#define CLIM_LEVEL 76       //2.4A
#define CLIM_LEVEL 86       //2.7A
//#define CLIM_LEVEL 95       //3.0A Runda fan setting
//#define CLIM_LEVEL 105       //3.3A
// I_LIM (A) = 4A * CLIM_LEVEL/127
i2c_mbed_fpga i2c;
DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
DigitalOut led4(LED4);
DigitalOut fpga_rstb(p27);   //p27 of MBED is connectted to p22 of the FPGA CMOD for 90415FPGA_EVB2

/* Main function */
int main() {
    char i2cBuffer[3];
    char i2cTestPassFlag = 0;
    char i2cRamDumpFlag = 0;
    char i2cTestPulseDebugFlag = 0;
    
    /*init funcitons*/
    main_init();  /*reset the FPGA!*/ 
    
    /*control codes*/
      wait_ms(2000);   /* keep FPGA in reset for 5s, to let motor stop*/
      enbale_fpga();   /* enable FPGA */ 
      wait_ms(40);      
      //*(i2cBuffer+0) = (char)(I2C_STATE)& 0xff;
      //i2c.i2c_word_read(i2cBuffer);
      //uart_print(i2cBuffer);
      //wait_ms(1);
      i2c.i2c_config_mode_entry(); 
      wait_ms(1);
      i2c.i2c_mlx_mode_entry();
      wait_ms(1);
      i2c.i2c_clear_spd_ctrl();
      wait_ms(1);
      i2c.i2c_set_comm_i_thres(0x0fff,1);  //note, 0fff is -1 signed
      wait_ms(1);
      i2c.i2c_set_comm_di_thres(0,1);
      wait_ms(1);
      i2c.i2c_set_fall_time_blank(4);
      wait_ms(1);
      i2c.i2c_set_50k_pwm(1);    
      wait_ms(1);        
      //i2c.i2c_set_position_pulse_width(3,5); /* Runda unsigned int mantisaa_2b, unsigned int exponent_3b*/
      i2c.i2c_set_position_pulse_width(2,4); /* VGA unsigned int mantisaa_2b, unsigned int exponent_3b */
      wait_ms(1);
      i2c.i2c_set_position_flat(3,7);
      wait_ms(1);
//    i2c.i2c_set_position_duty(3);
//    wait_ms(10);
//    i2c.i2c_set_position_anti_cog(0);
//    wait_ms(10);
//    i2c.i2c_set_start_up_duty(3);
//    wait_ms(10);
      //i2c.i2c_set_start_up_pulse_width(5,6); /*Runda(unsigned int mantisaa_3b, unsigned int exponent_3b*/
      i2c.i2c_set_start_up_pulse_width(3,5); /* VGA (unsigned int mantisaa_3b, unsigned int exponent_3b*/
      wait_ms(1);
      //i2c.i2c_set_start_up_flat(5,6); /*Runda(unsigned int mantisaa_3b, unsigned int exponent_3b*/
      i2c.i2c_set_start_up_flat(3,6); /*VGA (unsigned int mantisaa_3b, unsigned int exponent_3b*/
      wait_ms(1);
      i2c.i2c_set_start_up_num_comm(1);
//    wait_ms(10);
//    //led4 = i2c.i2c_set_soft_start_up(1, 1, 0, 1, 1);
//    wait_ms(10);
//    i2c.i2c_clear_spd_ctrl();
      wait_ms(1);
      i2c.i2c_set_loop_mode(0);
      wait_ms(1);
      i2c.i2c_set_curve_type(3);
      wait_ms(1);
      i2c.i2c_set_dc_ini(2);
      wait_ms(1);
      i2c.i2c_set_dc_sr(0);
      wait_ms(1);
      i2c.i2c_force_rough_reg(0);
      wait_ms(1);
      i2c.i2c_set_rough_gain(0);
      wait_ms(1);
      i2c.i2c_set_ehp_reg_gain(0);
      wait_ms(1);
      i2c.i2c_set_clim_start_up(CLIM_LEVEL);
      wait_ms(1);
      i2c.i2c_set_clim_brake(CLIM_LEVEL);
      wait_ms(1);
      i2c.i2c_set_clim_run_time(CLIM_LEVEL);
      wait_ms(1);

      
      if(i2cRamDumpFlag == 1){
      /*dump calibration data start*/
        i2c.i2c_mlx_mode_entry();
        wait_ms(1);
        *(i2cBuffer+0) = (char)(I2C_STATUS)& 0xff;
        i2c.i2c_word_read(i2cBuffer);
        uart_print(i2cBuffer);
        
        if( i2cBuffer[2] == 0x0C ){
            uart_I2C_test_pass();
            i2cTestPassFlag = 1;
            led4 = 0;
        }
        else {
            uart_I2C_test_fail();
            i2cTestPassFlag = 0;
            led4 = 1;
        }
        
        if(i2cTestPassFlag == 1){
            char i;
            for(i=0;i<=RAM_END_ADDR;i++) {
                *(i2cBuffer+0) = i;
                i2c.i2c_word_read(i2cBuffer);
                uart_print(i2cBuffer);
                wait_ms(300);
            }  
            uart_ram_dump_finish();
        }    
      }
      
      wait_ms(1);
      if(i2cTestPulseDebugFlag == 1) {
        i2c.i2c_mlx_mode_entry();
        wait_ms(1);
        *(i2cBuffer+0) = (char)(I2C_STATUS)& 0xff;
        i2c.i2c_word_read(i2cBuffer);
        uart_print(i2cBuffer);
        
        if( i2cBuffer[2] == 0x0C ){
            uart_I2C_test_pass();
            i2cTestPassFlag = 1;
            led4 = 0;
        }
        else {
            uart_I2C_test_fail();
            i2cTestPassFlag = 0;
            led4 = 1;
        }
        
        if(i2cTestPassFlag == 1){
            i2c.i2c_enable_pules_debug_mode(1);
            wait_ms(1);
        }
      }
      
      i2c.i2c_skip_app_copy();
      wait_ms(1);
      i2c.i2c_soft_reset();        
      i2c.wait_for_idle_state();
      wait_ms(3);
      i2c.i2c_mlx_mode_entry();
      wait_ms(1);
      i2c.i2c_set_open_loop_duty(0xffff); /*100%*/
      //i2c.i2c_set_open_loop_duty(0xf850); /*97%*/  
      if(i2cTestPulseDebugFlag == 1) {
        wait_ms(100);        
        char i;
        for(i=0xEA;i<=0xEF;i++) {
            *(i2cBuffer+0) = i;
            i2c.i2c_word_read(i2cBuffer);
            uart_print(i2cBuffer);
            wait_ms(300);
        }  
        uart_ram_dump_finish();
      }
    led2 = 1;
    /*enter infinite loop, motor should be running*/
    
//    scpi_def_init();

    SCPI_Init(&scpi_context);
    char* cmd1 = "TARG:DUTY 50\n";
    SCPI_Input(&scpi_context, cmd1, strlen(cmd1));
    char* cmd2 = "TARG:DUTY?\n";
    SCPI_Input(&scpi_context, cmd2, strlen(cmd2));
    
    while (1) {
        char buffer[50];
        uart_read(buffer);
        SCPI_Input(&scpi_context, buffer, strlen(buffer));
    }

}